@@ -1,7 +1,23 @@
+from __future__ import annotations
 import functools
+import sys
+from typing import (
+    Any,
+    Callable,
+    Container,
+    Protocol,
+    SupportsFloat,
+    SupportsIndex,
+    TypeVar,
+    Union,
+)
+if sys.version_info < (3, 10):  # pragma: no cover
+    from typing_extensions import ParamSpec
+else:
+    from typing import ParamSpec
 
 import pytest
-
+import math
 from .check_log import log_failure
 
 __all__ = [
@@ -14,6 +30,8 @@ __all__ = [
     "is_false",
     "is_none",
     "is_not_none",
+    "is_nan",
+    "is_not_nan",
     "is_in",
     "is_not_in",
     "is_instance",
@@ -25,16 +43,37 @@ __all__ = [
     "less",
     "less_equal",
     "between",
+    "between_equal",
     "check_func",
+    "fail",
 ]
 
 
-def check_func(func):
+_P = ParamSpec("_P")
+_T = TypeVar("_T")
+
+class _ComparableGreaterThan(Protocol):
+    def __gt__(self, other: Any, /) -> bool: ... # pragma: no cover
+
+
+class _ComparableGreaterThanOrEqual(Protocol):
+    def __ge__(self, other: Any, /) -> bool: ... # pragma: no cover
+
+
+class _ComparableLessThan(Protocol):
+    def __lt__(self, other: Any, /) -> bool: ... # pragma: no cover
+
+
+class _ComparableLessThanOrEqual(Protocol):
+    def __le__(self, other: Any, /) -> bool: ... # pragma: no cover
+
+
+def check_func(func: Callable[_P, _T]) -> Callable[_P, bool]:
     @functools.wraps(func)
-    def wrapper(*args, **kwds):
+    def wrapper(*args: _P.args, **kwargs: _P.kwargs) -> bool:
         __tracebackhide__ = True
         try:
-            func(*args, **kwds)
+            func(*args, **kwargs)
             return True
         except AssertionError as e:
             log_failure(e)
@@ -43,11 +82,11 @@ def check_func(func):
     return wrapper
 
 
-def assert_equal(a, b, msg=""):  # pragma: no cover
+def assert_equal(a: object, b: object, msg: str = "") -> None:  # pragma: no cover
     assert a == b, msg
 
 
-def equal(a, b, msg=""):
+def equal(a: object, b: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if a == b:
         return True
@@ -56,7 +95,7 @@ def equal(a, b, msg=""):
         return False
 
 
-def not_equal(a, b, msg=""):
+def not_equal(a: object, b: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if a != b:
         return True
@@ -65,7 +104,7 @@ def not_equal(a, b, msg=""):
         return False
 
 
-def is_(a, b, msg=""):
+def is_(a: object, b: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if a is b:
         return True
@@ -74,7 +113,7 @@ def is_(a, b, msg=""):
         return False
 
 
-def is_not(a, b, msg=""):
+def is_not(a: object, b: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if a is not b:
         return True
@@ -83,7 +122,7 @@ def is_not(a, b, msg=""):
         return False
 
 
-def is_true(x, msg=""):
+def is_true(x: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if bool(x):
         return True
@@ -92,7 +131,7 @@ def is_true(x, msg=""):
         return False
 
 
-def is_false(x, msg=""):
+def is_false(x: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if not bool(x):
         return True
@@ -101,7 +140,7 @@ def is_false(x, msg=""):
         return False
 
 
-def is_none(x, msg=""):
+def is_none(x: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if x is None:
         return True
@@ -110,7 +149,7 @@ def is_none(x, msg=""):
         return False
 
 
-def is_not_none(x, msg=""):
+def is_not_none(x: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if x is not None:
         return True
@@ -119,7 +158,25 @@ def is_not_none(x, msg=""):
         return False
 
 
-def is_in(a, b, msg=""):
+def is_nan(a: SupportsFloat | SupportsIndex, msg: str = "") -> bool:
+    __tracebackhide__ = True
+    if math.isnan(a):
+        return True
+    else:
+        log_failure(f"check {a} is NaN", msg)
+        return False
+
+
+def is_not_nan(a: SupportsFloat | SupportsIndex, msg: str = "") -> bool:
+    __tracebackhide__ = True
+    if not math.isnan(a):
+        return True
+    else:
+        log_failure(f"check {a} is not NaN", msg)
+        return False
+
+
+def is_in(a: _T, b: Container[_T], msg: str = "") -> bool:
     __tracebackhide__ = True
     if a in b:
         return True
@@ -128,7 +185,7 @@ def is_in(a, b, msg=""):
         return False
 
 
-def is_not_in(a, b, msg=""):
+def is_not_in(a: _T, b: Container[_T], msg: str = "") -> bool:
     __tracebackhide__ = True
     if a not in b:
         return True
@@ -137,7 +194,9 @@ def is_not_in(a, b, msg=""):
         return False
 
 
-def is_instance(a, b, msg=""):
+_TypeTuple = Union[type, tuple['_TypeTuple', ...]]
+
+def is_instance(a: object, b: _TypeTuple, msg: str = "") -> bool:
     __tracebackhide__ = True
     if isinstance(a, b):
         return True
@@ -146,7 +205,7 @@ def is_instance(a, b, msg=""):
         return False
 
 
-def is_not_instance(a, b, msg=""):
+def is_not_instance(a: object, b: _TypeTuple, msg: str = "") -> bool:
     __tracebackhide__ = True
     if not isinstance(a, b):
         return True
@@ -155,7 +214,9 @@ def is_not_instance(a, b, msg=""):
         return False
 
 
-def almost_equal(a, b, rel=None, abs=None, msg=""):
+def almost_equal(
+    a: object, b: object, rel: Any = None, abs: Any = None, msg: str = ""
+) -> bool:
     """
     For rel and abs tolerance, see:
     See https://docs.pytest.org/en/latest/builtin.html#pytest.approx
@@ -168,7 +229,9 @@ def almost_equal(a, b, rel=None, abs=None, msg=""):
         return False
 
 
-def not_almost_equal(a, b, rel=None, abs=None, msg=""):
+def not_almost_equal(
+    a: object, b: object, rel: Any = None, abs: Any = None, msg: str = ""
+) -> bool:
     """
     For rel and abs tolerance, see:
     See https://docs.pytest.org/en/latest/builtin.html#pytest.approx
@@ -181,7 +244,7 @@ def not_almost_equal(a, b, rel=None, abs=None, msg=""):
         return False
 
 
-def greater(a, b, msg=""):
+def greater(a: _ComparableGreaterThan, b: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if a > b:
         return True
@@ -190,7 +253,7 @@ def greater(a, b, msg=""):
         return False
 
 
-def greater_equal(a, b, msg=""):
+def greater_equal(a: _ComparableGreaterThanOrEqual, b: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if a >= b:
         return True
@@ -199,7 +262,7 @@ def greater_equal(a, b, msg=""):
         return False
 
 
-def less(a, b, msg=""):
+def less(a: _ComparableLessThan, b: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if a < b:
         return True
@@ -208,7 +271,7 @@ def less(a, b, msg=""):
         return False
 
 
-def less_equal(a, b, msg=""):
+def less_equal(a: _ComparableLessThanOrEqual, b: object, msg: str = "") -> bool:
     __tracebackhide__ = True
     if a <= b:
         return True
@@ -217,7 +280,9 @@ def less_equal(a, b, msg=""):
         return False
 
 
-def between(b, a, c, msg="", ge=False, le=False):
+def between(
+    b: Any, a: Any, c: Any, msg: str = "", ge: bool = False, le: bool = False
+) -> bool:
     __tracebackhide__ = True
     if ge and le:
         if a <= b <= c:
@@ -243,3 +308,18 @@ def between(b, a, c, msg="", ge=False, le=False):
         else:
             log_failure(f"check {a} < {b} < {c}", msg)
             return False
+
+
+def between_equal(
+    b: _ComparableLessThanOrEqual,
+    a: _ComparableLessThanOrEqual,
+    c: object,
+    msg:str = "",
+) -> bool:
+    __tracebackhide__ = True
+    return between(b, a, c, msg, ge=True, le=True)
+
+
+def fail(msg: str) -> None:
+    __tracebackhide__ = True
+    log_failure(msg)
